#!/usr/bin/perl -w

# Nom du programme:clean.pl 

# read a fasta file into a hachage cleans it (non-ATGCN are changed into N),
# lowercase are changed to upper case, Ns are deleted from the begining
# and at the end of each sequences, Vector contaminations are checked are removed)

# produces a fasta file with the clean sequences (name.clean)
# produces a fasta file with the clean sequences with microsatellites masked by Ns (name.WOV)
# produces two info files on the cleaning done (*.inf fixed format, *.infx semicolumns as separators)
# produces a *.vec file with the result of blasting the sequences againts the univec vector base


  use warnings;
  use strict;
  use sub_ms_group;

  my $USAGE = "USAGE : $0 filename_with_pathway folder_for_the_outfiles minimum_number_of_repeats_for_2-3-4-5-6-bp_motifs minimum_number_of_repeats_for_single-bp_motifs E_value_for checking_vector_contamination pathway_for_Blast\n\n";

  unless ( scalar@ARGV == 6 )
  {
      print $USAGE;
      exit;
  }

  my $filename = $ARGV[0];
  my $outfolder =  $ARGV[1];
  my $min_repeat = $ARGV[2];
  my $min_repeat_single = $ARGV[3];
  my $Evector =  $ARGV[4];
  my $path_blast =  $ARGV[5];

  unless (open(SEQ, $filename))
    {
      print "Cannot open $filename\n";
      exit;
    }
  my @seq = <SEQ>;
  close(SEQ);

# delete empty lines and concatanate the diff lines of each sequences, put sequence into a hachage
  my %seq_hach = clean_file_2(@seq);
#  print_hachage(\%seq_hach);

 my $new_filename = check_code_overlap(\%seq_hach, $outfolder);
 unless ($new_filename eq '')
 {
     $filename = $new_filename;
 }


# delete spaces, check the presence of non-AGTCN, change all other
# characters into N
# codes are untouched
  my %NON_ATGC = change_letters(\%seq_hach);
#  print_hachage(\%NON_ATGC);

# make a hachage with the length of each sequence
  my %orig_length  = length_sequences(\%seq_hach);
#  print_hachage(\%orig_length);
  
# delete Ns at the begining and end of the sequence (if if more than 2 N/10) 
# The number of bases deleted from the begining and from the end of the 
# sequence are strored in N_hach
# seq_hach is overwritten with the shortened sequences
  my %N_hach = delete_Ns(\%seq_hach);
#  print_hachage(\%N_hach);

# find microsat motifs and positions, stores them into ms_hachage 
  my %ms_hachage = find_microsat_2(\%seq_hach, $min_repeat, $min_repeat_single);
#  print_hachage(\%ms_hachage);

# eliminate sequences without microsat motif
  my %seq_hach_Wmicrosat = delete_seq_WOmicrosat(\%seq_hach, \%ms_hachage);
#  print_hachage(\%seq_hach_Wmicrosat);

# make a fasta file with the clean sequences (from seq_hach_Wmicrosat into name.WON)
  $filename =~ /\/[a-z0-9_]+\./i; 
  $filename = $outfolder.$&.'WON';
  make_fasta_file(\%seq_hach_Wmicrosat, $filename);
#  print "\nCleaned sequences are saved into $filename file\n(sequences WO microsat are eliminated, Ns are deleted from beg and end\n\n";

# blasts cleaned file (name.WON) againts Univec.fas
# blast results are saved into name.vec
  call_blast_vector($filename, $Evector, $path_blast);

# delete *.WON
delete_file($filename);

# open txt file with the results of the blast
  $filename =~ s/\.[a-z0-9]+/\.vec/i;
print "$filename: The results of a blast against the Univec vector base\n";
  unless ( open(BLASTRES, $filename) )
    {
       print "Cannot open $filename!\n";
    }

#read the blast results for one sequence
   my %vector = ();
   my $res_seq = '';
   while ($res_seq = one_seq_result('BLASTRES'))
   {
       # avoide last scalar with only  a few characters 
       if ($res_seq !~ /Query/) 
       {
	   next;
       }

      # extract from the blast results of a given sequence the name of the 
      # most far reching vector at the begining and its position, and the 
      # name of the most far reaching vector at the end and its positions        
      my ($code, $vector_beg, $vector_name_beg, $vector_end, $vector_name_end) = extract_vector_res(\$res_seq, \%ms_hachage);

      # produce a hachage (vector) with the code of the sequence and the name 
      # and positions of the most far reaching vectors at the begining and at the end 
      my $vector_pos = "POSBEG: $vector_beg $vector_name_beg POSEND: $vector_end $vector_name_end";
      $vector{$code} = $vector_pos;

      # cut off vector contaminantion and over-write seq_hach with the clean sequences  
      my $seqtemp = $seq_hach_Wmicrosat{$code};
      $seq_hach_Wmicrosat{$code} = clean_seq_from_vector($vector_beg, $vector_end, $seqtemp);   
   }
   close BLASTRES;

#  print "Vector positions are written into vector (hachage)\nSequences without vector contaminations are writen in seq_hach_Wmicrosat\n";
#  print_hachage(\%vector);

# make a hachage with the length of each sequence
  my %clean_length  = length_sequences(\%seq_hach_Wmicrosat);
#  print_hachage(\%clean_length);

#Re-calculate Ms position after vector cut
 recalculate_MS_pos_1(\%ms_hachage, \%vector);
# print_hachage(\%ms_hachage);

  $filename =~ s/\.[a-z0-9]+/\.info/i;
  write_info_separateur(\%orig_length, \%NON_ATGC, \%N_hach, \%vector, \%clean_length, \%ms_hachage, $filename);
print "$filename: Information on the cleaned sequences\n";


 #  make a fasta file with a newly cleaned sequences MS not masked (WO vector, name.clean)
   $filename =~ s/\.[a-z0-9]+/\.clean/i;
   make_fasta_file(\%seq_hach_Wmicrosat, $filename);
   print "$filename: Microsatellite containing clean sequences in fasta format\n";

#  make a fasta file with a newly cleaned sequences (WO vector, name.WOV)
#  microsatellites are masked (input file for identifying grouped sequences)
   $filename =~ s/\.[a-z0-9]+/\.WOV/i;
   make_fasta_file_masqued(\%seq_hach_Wmicrosat, \%ms_hachage, $filename);
   print "$filename: Microsatellite containing clean sequences in fasta format; Microsatellites are masked by Ns\n";
   


  exit;


##########################################################

sub one_seq_result
{
#    read the result of the blast for one sesquence from file

    my $filename = @_; 


    my $sep_entree = $/;
    $/ = "Posted date";
    my $res_seq = <BLASTRES>;
    $/ = $sep_entree;
    return $res_seq;
}

##############################################################
